* analysis_sample_all_vars.do
* David Card, Ana Rute Cardoso, Joerg Heining, Patrick Kline
* jole 2017

set more 1
program drop _all
capture log close
log using ..\logs\analysis_sample_all_vars.log, replace


*** firms with log VA 05-09, log sales 04-10, and empl 04-10, never missing

use ..\data\firms_1999_2012.dta, clear
keep if ano>=2004 & ano<=2010
ren idempadd idfirm
ren ano year

gen tmpVA=1    if lrVA!=. & sizefQP!=. & year>=2005 & year<=2009
gen tmpsales=1 if lrsales!=. & sizefQP!=. & year>=2004 & year<=2010

bys idfirm: egen tmptotVA=total(tmpVA)
bys idfirm: egen tmptotsales=total(tmpsales)
keep if tmptotVA==5 & tmptotsales==7

* industry, region

foreach v in caem5 ind nutsm3 lisbon oporto {
	sort idfirm year
	by idfirm: gen `v'_1st = `v'[1]
	drop `v'
}
gen ind3_1st = int(caem5_1st/100)

* per capita Sales and VA, winzorize outliers yearly (=p2 or =p98 if smaller or larger, respectively)

gen lsalespe = log(rsales / sizefQP)
gen lVApe    = log(rVA / sizefQP)

foreach v in lVApe lsalespe {
	foreach p in 2 98 {
     		bys year: egen p`p'`v' = pctile(`v'), p(`p')
	}
	gen `v'_winz=`v'
	replace `v'_winz = p2`v'  if `v'<p2`v'
	replace `v'_winz = p98`v' if `v'>p98`v' & `v'!=.
	drop `v' p2* p98*
}

* average lVApe, lsalespe

bys idfirm: egen avv0509 = mean(lVApe_winz) if year>=2005 & year<=2009
bys idfirm: egen avs0410 = mean(lsalespe_winz) if year>=2004 & year<=2010
bys idfirm: egen avs0509 = mean(lsalespe_winz) if year>=2005 & year<=2009

* assign each average to obs in all years

bys idfirm: egen avlVApe0509 = mean(avv0509)
bys idfirm: egen avlsalespe0410 = mean(avs0410)
bys idfirm: egen avlsalespe0509 = mean(avs0509)

* change in lVApe lsalespe, winzorize outliers (=p2 or =p98 if smaller or larger, respectively)

sort idfirm year
gen DlVApe05_09 = lVApe_winz - L4.lVApe_winz if year==2009
gen Dlsalespe05_09 = lsalespe_winz - L4.lsalespe_winz if year==2009
gen Dlsalespe04_10 = lsalespe_winz - L6.lsalespe_winz if year==2010

gen DlVApe05_09w = DlVApe05_09
gen Dlsalespe05_09w = Dlsalespe05_09
gen Dlsalespe04_10w = Dlsalespe04_10

foreach p in 2 98 {
	egen p`p'DlVApe05_09 = pctile(DlVApe05_09), p(`p')
     	egen p`p'Dlsalespe05_09 = pctile(Dlsalespe05_09), p(`p')
     	egen p`p'Dlsalespe04_10 = pctile(Dlsalespe04_10), p(`p')
}

replace DlVApe05_09w = p2DlVApe05_09  if DlVApe05_09<p2DlVApe05_09
replace DlVApe05_09w = p98DlVApe05_09 if DlVApe05_09>p98DlVApe05_09 & DlVApe05_09!=.
bys idfirm: egen DlVApe05_09_winz = mean(DlVApe05_09w)
drop DlVApe05_09 DlVApe05_09w

foreach y_y in 05_09 04_10 {
	replace Dlsalespe`y_y'w = p2Dlsalespe`y_y'  if Dlsalespe`y_y'<p2Dlsalespe`y_y'
	replace Dlsalespe`y_y'w = p98Dlsalespe`y_y' if Dlsalespe`y_y'>p98Dlsalespe`y_y' & Dlsalespe`y_y'!=.
	bys idfirm: egen Dlsalespe`y_y'_winz = mean(Dlsalespe`y_y'w)
	drop Dlsalespe`y_y' Dlsalespe`y_y'w
}
keep year idfirm ind_1st ind3_1st lisbon_1st oporto_1st nutsm3_1st avl* D*
save ..\data\firms_analysis_rolefirmCKC_2004_2010.dta, replace


*** read estimates AKM models 2002-09: males (2 skills, all), females

program define readakm
   import delimited idworker idfirm year pe_`2' fe_`2' xb_`2' r_`2' using ..\data\AKMeffs`1'.txt, clear
   recast double idworker idfirm 
   recode idfirm 88888888 = 653075

   label variable pe_`2' "person effect, `2'"
   save ..\data\tmppersoneffs_`2'.dta, replace

   collapse fe_`2' pe_av`2'=pe_`2' (count) Nobs_`2'_connected=pe_`2', by(idfirm)
   label variable fe_`2' "firm effect, `2' skill"
   save ..\data\tmpfirmeffs_`2'.dta, replace
end

readakm  1_byskill mlow
readakm  2_byskill mhigh
readakm  5         m
readakm  6         f


* merge worker effects 

use  ..\data\tmppersoneffs_mlow.dta, clear
merge 1:1 idworker year using ..\data\tmppersoneffs_mhigh.dta, update
assert _merge!=3
drop _merge
merge 1:1 idworker year using ..\data\tmppersoneffs_m.dta
drop _merge
merge 1:1 idworker year using ..\data\tmppersoneffs_f.dta
assert _merge!=3
save ..\data\AKMperson_effs_skill_all.dta, replace

* merge firm effects

use ..\data\tmpfirmeffs_mlow.dta, clear
merge 1:1 idfirm using ..\data\tmpfirmeffs_mhigh.dta, update
drop _merge
merge 1:1 idfirm using ..\data\tmpfirmeffs_m.dta, update
drop _merge
merge 1:1 idfirm using ..\data\tmpfirmeffs_f.dta, update

foreach g in mlow mhigh m f {
   recode Nobs_`g'_connected .=0
}
save ..\data\AKMfirm_effs_skill_all.dta, replace


*** firms all vars 

use ..\data\firms_analysis_rolefirmCKC_2004_2010.dta, clear
merge m:1 idfirm using ..\data\AKMfirm_effs_skill_all.dta
keep if _merge==3 | _merge==1 /* merge=1 not in connected set; merge=2 not in set lVape 05-09 and lsalespe 04-10 not miss */

* connected sets firm belongs to in analysis sample 2005-09 

gen setfirmsk0509=3 if fe_mlow!=. & fe_mhigh!=.
replace setfirmsk0509=1 if fe_mlow!=. & fe_mhigh==.
replace setfirmsk0509=2 if fe_mlow==. & fe_mhigh!=.
label values setfirmsk0509 setfirmsklb
label variable setfirmsk0509 "firm in connected set 05-09 (low,high,both skills)"

gen setfirm0509=3 if fe_m!=. & fe_f!=.
replace setfirm0509=1 if fe_m!=. & fe_f==.
replace setfirm0509=2 if fe_m==. & fe_f!=.
label values setfirm0509 setfirmlb
label variable setfirm0509 "firm in connected set(s) 05-09 (m,f,both)"

keep year idfirm ind_1st ind3_1st lisbon_1st oporto_1st nutsm3_1st avl* D* fe_* pe_av* Nobs_* setfirm*
save ..\data\firms_all_info_rolefirmCKC_04_10.dta, replace


*** workers all vars 

use  ..\data\workers_analysis_02_09.dta, clear

bys idworker: egen edmode = mode(educ), maxmode
drop if age-edmode-6<2
keep if year>=2005 & year<=2009 

merge m:1 idfirm year using ..\data\firms_all_info_rolefirmCKC_04_10.dta, update
keep if _merge==3  
drop _merge

merge 1:1 year idworker idfirm using ..\data\AKMperson_effs_skill_all.dta, update
keep if _merge==3 | _merge==1

gen skillmode=1 if edmode<12
replace skillmode=2 if edmode>=12 & edmode!=.

keep year idworker idfirm gender age edmode skillmode lwage Dlwage ind_1st ind3_1st lisbon_1st oporto_1st nutsm3_1st avl* D* pe_* fe_* xb_* r_* setworker* setfirm* Nobs_* 
save ..\data\workers_all_info_rolefirmCKC_05_09.dta, replace

log close
