* analysis_sample_workers.do 
* David Card, Ana Rute Cardoso, Joerg Heining, Patrick Kline
* jole 2017

set more 1
program drop _all
capture log close
log using ..\logs\analysis_sample_workers.log, replace


*** workers 2002-09

use idtrab ano gender datanasc educ using ..\data\worker_demog02_09.dta, clear
merge idtrab ano using ..\data\worker_hire02_09.dta
recast double idemp
recast double idtrab

* age

gen birthmonth=datanasc-(int(datanasc/100))*100
gen birthyear=int(datanasc/100)
gen birthddate=mdy(birthmonth,01,birthyear)
format birthddate %td
gen age=int((mdy(10,01,ano)-birthddate)/365)

* date hire

gen hiremonth=datehire-(int(datehire/100))*100
gen hireyear=int(datehire/100)
gen hireddate=mdy(hiremonth,01,hireyear)
format hireddate %td

keep     ano idtrab idemp gender age educ hireddate
compress ano              gender age educ hireddate
save ..\data\tmpworker_2002_09.dta, replace


* yearly employment data

foreach yr of numlist 2002(1)2009 {
   use ..\data\trabs`yr'_SelectionVars.dta, clear
   bys idtrab idemp: keep if _n==1
   bys idemp: gen sizeQP=_N
   
   recode hnorm 0=.
   recode wbase 0=.
   gen wh=(wbase+wregul)/hnorm
   
   gen ft=1 if hnorm>=35*4 & hnorm!=.
   replace ft=0 if hnorm<35*4

   gen occup1=int(occup3/100)

   keep     ano idtrab idemp wbase wh hnorm hextra ft occup3 occup1 sizeQP
   compress ano              wbase wh hnorm hextra ft occup3 occup1 sizeQP
   save ..\data\tmpempl`yr'.dta, replace
}

foreach yr of numlist 2002(1)2008 {
   append using ..\data\tmpempl`yr'.dta
}

merge 1:1 idtrab idemp ano using ..\data\tmpworker_2002_09.dta
keep if _merge==3


* deflate w and drop outliers

program define deflateIPC
	gen     lr`1'=log(`1')+0.1518 if ano==2002
	replace lr`1'=log(`1')+0.1193 if ano==2003
	replace lr`1'=log(`1')+0.0956 if ano==2004
	replace lr`1'=log(`1')+0.0729 if ano==2005
	replace lr`1'=log(`1')+0.0423 if ano==2006
	replace lr`1'=log(`1')+0.0176 if ano==2007
	replace lr`1'=log(`1')-0.0080 if ano==2008
	replace lr`1'=log(`1')+0.0000 if ano==2009
	gen r`1'=exp(lr`1')
end
deflateIPC wh

gen tmpdrop=.
foreach yr of numlist 2002(1)2009 {
   qui sum rwh if ano==`yr', detail
   replace tmpdrop=1 if (rwh>3*r(p99) | rwh<0.80*r(p1)) & rwh!=. & ano==`yr' 
}
egen tmpdrop2=sum(tmpdrop), by(idtrab)
drop if tmpdrop2>0
drop tmp*

xtset idtrab ano
gen Dlwage = D.lrwh
gen tmpdrop=1 if Dlwage<-1 | (Dlwage>1 & Dlwage!=.)
egen tmpdrop3=sum(tmpdrop), by(idtrab)
drop if tmpdrop3>0
drop tmp*


* wage-earner aged 19-65, potential experience>=2

keep if age>=19 & age<=65
keep if age-educ-6>1 & age-educ-6!=.
keep if lrwh!=.


* educ 0-4 years lumped together 

recode educ 0=4


* min w

gen wmin=.
replace wmin=348.01 if ano==2002
replace wmin=356.60 if ano==2003
replace wmin=365.60 if ano==2004
replace wmin=374.70 if ano==2005
replace wmin=385.90 if ano==2006
replace wmin=403.00 if ano==2007
replace wmin=426.00 if ano==2008
replace wmin=450.00 if ano==2009

 
* pink / blue occupation

egen sharefem_occup = mean(gender), by(occup3)
egen sharefem_occup_avperson = mean(sharefem_occup), by(idtrab)
quietly sum sharefem_occup, detail
gen pink=1     if sharefem_occup_avperson>=r(p50) & sharefem_occup_avperson!=.
replace pink=0 if sharefem_occup_avperson<r(p50)  & sharefem_occup_avperson!=.


* wage of co-workers

egen wtot = total(lrwh), by(ano idemp)
gen wtot_peers= wtot-lrwh
bys ano idemp: gen Npeers=_N-1
gen lwpeers = wtot_peers / Npeers


* firm size analysis sample

bys idemp ano: gen sizeanalysis=_N 


* labels

ren ano year
ren idtrab idworker
ren idemp idfirm
ren lrwh lwage

label variable idworker    "worker id"
label variable idfirm      "firm id"
label variable gender      "female"
label variable educ        "education completed (yrs)"
label variable hireddate   "date hire (stata format)"
label variable lwage       "(log) real hourly wage"
label variable Dlwage      "change in (log) real hourly wage"
label variable wmin        "monthly min wage"
label variable wbase       "monthly base wage"
label variable hnorm       "normal monthly hours work"
label variable hextra      "overtime monthly hours work"
label variable occup3      "occupation (3 digits)"
label variable occup1      "occupation (1 digit)"
label variable pink        "worker in predomin. female occupation"
label variable lwpeers     "av. (log) wage co-workers"

keep     year idworker idfirm gender age educ wbase wmin lwage Dlwage hnorm hextra ft occup3 occup1 pink lwpeers hireddate sizeQP sizeanalysis
compress year                 gender age educ wbase wmin lwage Dlwage hnorm hextra ft occup3 occup1 pink lwpeers hireddate sizeQP sizeanalysis 
sum
save ..\data\workers_analysis_02_09.dta, replace


*** export to run AKMs

use idworker idfirm year lwage gender age educ pink sizeanalysis using ..\data\workers_analysis_02_09.dta, clear

ren idworker id
ren idfirm firmid
ren lwage y
ren size fsize_ckc

recode firmid  653075 = 88888888

keep     id year firmid y gender age educ pink fsize_ckc  
order    id year firmid y gender age educ pink fsize_ckc  
compress    year        y gender age educ pink fsize_ckc  
sort id year
save ..\data\tmpPT2002_2009.dta, replace

bys gender: sum

program define toakm
   use ../data/tmpPT2002_2009.dta if gender==`1', clear
   drop gender
   outfile using ../data/PT_2002_2009_`2'_commadelimited, wide comma replace
end

toakm 0 males
toakm 1 females

log close

