path(path,'~/matlab_bgl/'); %path to the matlabBGL files 
%note: matlab BGL obtained from http://www.mathworks.com/matlabcentral/fileexchange/10922
system('rm AKM_portugal_norm.log')
diary('AKM_portugal_norm.log')
clear


for i=1:5

if i==1
    mu=40
elseif i==2
    mu=50
elseif i==3
    mu=30
elseif i==4
    mu=0
elseif i==5
    disp('Gaussian Basis!')
end
    
g='males';

l='all';

s=['Loading data for ' g ' , ' l '...'];
disp(s);
%LOAD DATA    
s=['PT_2002_2009_' g '_commadelimited.raw'];
data=importdata(s);

%impose an initial sample restriction
year=data(:,2);
age=data(:,6);
exp=age-data(:,7)-6;

sel=(age>=19)&(age<=65)&(exp>=2);


data=data(sel,:);

id=data(:,1);
year=data(:,2);
firmid=data(:,3);
y=data(:,4);
birth=data(:,5);
age=data(:,6);
educ=data(:,7);
fsize=round(data(:,8));
clear data

lagid=[NaN;id(1:end-1)];
sameid=lagid==id;
lagfirmid=[NaN;firmid(1:end-1)];
lagfirmid(~sameid)=NaN;


educ(educ==0)=0;
educ(educ==4)=0;
educ(educ==6)=1;
educ(educ==9)=2;
educ(educ==12)=3;
educ(educ==16)=4;

fprintf('\n')
disp('Recoded education tab')
tabulate(educ)
disp('Note: 0 and 4 years of schooling lumped together into group 0')




firmid_old=firmid;
id_old=id;

fprintf('\n')
if i==1
    big=fsize==max(fsize);
    ref=min(firmid(big));
    s=['Reference firm is: ' int2str(ref)];
    disp(s)
    s=['Reference firm has minimum size: ' int2str(min(fsize(firmid==ref)))];
    disp(s)
    maxid=max(firmid);
    firmid(firmid==ref)=maxid+1;
    lagfirmid(lagfirmid==ref)=maxid+1;
end
if i>1
    disp('Double check..')
    s=['Reference firm has minimum size: ' int2str(min(fsize(firmid==ref)))];
    disp(s)
    firmid(firmid==ref)=maxid+1;
    lagfirmid(lagfirmid==ref)=maxid+1;
end
fprintf('\n')

lagfirmid(lagfirmid==-9)=NaN;


%RENAME
disp('Relabeling ids...')
N=length(y);
sel=~isnan(lagfirmid);

%relabel the firms
[firms,m,n]=unique([firmid;lagfirmid(sel)]);

firmid=n(1:N);
lagfirmid(sel)=n(N+1:end);


%relabel the workers
[ids,m,n]=unique(id);
id=n;

%initial descriptive stats
fprintf('\n')
disp('Some descriptive stats:')
s=['# of p-y obs: ' int2str(length(y))];
disp(s);
s=['# of workers: ' int2str(max(id))];
disp(s);
s=['# of firms: ' int2str(max(firmid))];
disp(s);

s=['mean wage: ' num2str(mean(y))];
disp(s)
s=['variance of wage: ' num2str(var(y))];
disp(s)
fprintf('\n')

%FIND CONNECTED SET
disp('Finding connected set...')
A=sparse(lagfirmid(sel),firmid(sel),1); %adjacency matrix
%make it square
[m,n]=size(A);
if m>n
    A=[A,zeros(m,m-n)];
end
if m<n
    A=[A;zeros(n-m,n)];
end
A=max(A,A'); %connections are undirected

[sindex, sz]=components(A); %get connected sets
idx=find(sz==max(sz)); %find largest set
s=['# of firms: ' int2str(length(A))];
disp(s);
s=['# connected sets:' int2str(length(sz))];
disp(s);
s=['Largest connected set contains ' int2str(max(sz)) ' firms'];
disp(s);
fprintf('\n')
clear A lagfirmid
firmlst=find(sindex==idx); %firms in connected set
sel=ismember(firmid,firmlst);

y=y(sel); firmid=firmid(sel); id=id(sel);
year=year(sel); id_old=id_old(sel); firmid_old=firmid_old(sel);
birth=birth(sel); educ=educ(sel); fsize=fsize(sel); N=length(y); 
age=age(sel);

disp('Relabeling ids again...')
%relabel the firms
[firms,m,n]=unique(firmid);
firmid=n;

%relabel the workers
[ids,m,n]=unique(id);
id=n;

%descriptive stats for connected set
fprintf('\n')
disp('Now restricted to the largest connected set');
s=['# of p-y obs: ' int2str(length(y))];
disp(s);
s=['# of workers: ' int2str(max(id))];
disp(s);
s=['# of firms: ' int2str(max(firmid))];
disp(s);

s=['mean wage: ' num2str(mean(y))];
disp(s)
s=['variance of wage: ' num2str(var(y))];
disp(s)
fprintf('\n')

%ESTIMATE AKM
disp('Building matrices...')
NT=length(y);
N=max(id);
J=max(firmid);

D=sparse(1:NT,id',1);
F=sparse(1:NT,firmid',1);

S=speye(J-1);
S=[S;sparse(-zeros(1,J-1))];  %N+JxN+J-1 restriction matrix 

%Build time trends
yrmin=min(year); yrmax=max(year);
educmin=min(educ); educmax=max(educ);

R=sparse(1:NT,(year-yrmin+1)+(yrmax-yrmin+1)*educ,1); %year effects by education
idx=1+(yrmax-yrmin+1)*educ;
R(:,idx)=[]; %drop first year effect in each education group


E=sparse(1:NT,educ+1,1);
%age=year-birth;
disp('Age tabulation')
tabulate(age)
if i<5
age=(age-mu)/40; %rescale to avoid big numbers
A=[bsxfun(@times,E,age.^2),bsxfun(@times,E,age.^3)]; %age cubic by education
end
if i==5
    A=bsxfun(@minus,age,[20:5:65]);
    G=normpdf(A/5);    
    A=[bsxfun(@times,E(:,1),G),bsxfun(@times,E(:,2),G),bsxfun(@times,E(:,3),G),bsxfun(@times,E(:,4),G),bsxfun(@times,E(:,5),G)];
end

Z=[R, A];

clear R E A


X=[D,F*S,Z];

disp('Running AKM...')
tic
xx=X'*X;
xy=X'*y;
L=ichol(xx,struct('type','ict','droptol',1e-2,'diagcomp',.2));
b=pcg(xx,xy,1e-12,1000,L,L');
toc
disp('Done')
clear xx xy L

%ANALYZE RESULTS
xb=X*b;
r=y-xb;

disp('Goodness of fit:')
dof=NT-J-N+1-size(Z,2)
RMSE=sqrt(sum(r.^2)/dof)

TSS=sum((y-mean(y)).^2);
R2=1-sum(r.^2)/TSS
adjR2=1-sum(r.^2)/TSS*(NT-1)/dof

ahat=b(1:N);
ghat=b(N+1:N+J-1);
bhat=b(N+J:end);
disp('check for problems with year effects. should report zero')
sum(bhat==0)

pe=D*ahat;
fe=F*S*ghat;
xb=X(:,N+J:end)*bhat;

clear D F X b

disp('Variance-Covariance of worker and firm effs (p-y weighted)');
cov(pe,fe)
disp('Correlation coefficient');
corr(pe,fe)
disp('Means of person/firm effs')
mean([pe,fe])

disp('Full Covariance Matrix of Components')
disp('    y      pe      fe      xb      r')
C=cov([y,pe,fe,xb,r])


%Report Age Profiles
disp('Age profile by education group')

A=[19:65]';
if i<5
    Anew=[((A-mu)/40).^2 ((A-mu)/40).^3];
    bnew=bhat(end-10:end);
    B=[bnew(1:5)'; bnew(6:10)'];
end
if i==5
    Anew=bsxfun(@minus,A,[20:5:65]);
    Anew=normpdf(Anew/5);
    bnew=bhat(end-50:end);
    B=[bnew(1:10), bnew(11:20), bnew(21:30), bnew(31:40), bnew(41:50)];
end


disp('ed group: 0 1 2 3 4') 
R=[A Anew*B]

%plot(A,R(:,2),A,R(:,3),A,R(:,4),A,R(:,5),A,R(:,6))



disp('Decomposition #1')
disp('var(y) = cov(pe,y) + cov(fe,y) + cov(xb,y) + cov(r,y)');
c11=C(1,1); c21=C(2,1); c31=C(3,1); c41=C(4,1); c51=C(5,1);
s=[num2str(c11) ' = ' num2str(c21) ' + ' num2str(c31) ' + ' num2str(c41) ' + ' num2str(c51)];
disp(s)
fprintf('\n')
disp('explained shares:    pe       fe       xb       r')
s=['explained shares: ' num2str(c21/c11) '  ' num2str(c31/c11) '  ' num2str(c41/c11) '  ' num2str(c51/c11)];
disp(s)

fprintf('\n')
disp('Decomposition #2')
disp('var(y) = var(pe) + var(fe) + var(xb) + 2*cov(pe,fe) + 2*cov(pe,xb) + 2*cov(fe,xb) + var(r)');
c11=C(1,1); c22=C(2,2); c33=C(3,3); c44=C(4,4); c55=C(5,5); 
c23=C(2,3); c24=C(2,4); c34=C(3,4);
s=[num2str(c11) ' = ' num2str(c22) ' + ' num2str(c33) ' + ' num2str(c44) ' + '  num2str(2*c23) ' + ' num2str(2*c24) ' + ' num2str(2*c34) ' + ' num2str(c55)];
disp(s)
fprintf('\n')
disp('explained shares:    pe      fe      xb   cov(pe,fe)   cov(pe,xb)   cov(fe,xb)   r')
s=['explained shares: ' num2str(c22/c11) '  ' num2str(c33/c11) '  ' num2str(c44/c11) '  ' num2str(2*c23/c11) '  ' num2str(2*c24/c11) '  ' num2str(2*c34/c11) '  ' num2str(c55/c11)];
disp(s)
fprintf('\n')


%joint distribution and separability
fedec = ceil(10 * tiedrank(fe) / length(fe));
pedec = ceil(10 * tiedrank(pe) / length(pe));
for j=1:10
    for k=1:10
        p(j,k)=mean((pedec==j)&(fedec==k));
        meanr(j,k)=mean(r.*(pedec==j).*(fedec==k))/p(j,k);
    end
end
disp('Joint distribution of effects (rows are deciles of pe, cols are deciles of fe)')
p
disp('Mean residual by pe/fe decile')
meanr

clear fedec pedec

%match effects
disp('Match Effects Model')
dig=ceil(max(log10(firmid)));
firmiddec=firmid./(10^dig);
matchid=id+firmiddec;
[matchnew,m,n]=unique(matchid);
matchid=n;

M=sparse(1:NT,matchid',1);
X=[M,Z];
xx=X'*X;
xy=X'*y;
L=ichol(xx,struct('type','ict','droptol',1e-2,'diagcomp',.1));
b=pcg(xx,xy,1e-10,1000,L,L');
r_match=y-X*b;
dof_match=NT-size(X,2)
RMSE_match=sqrt(sum(r_match.^2)/dof_match)
R2_match=1-sum(r_match.^2)/TSS
adjR2_match=1-sum(r_match.^2)/TSS*(NT-1)/dof_match
clear Z r_match b


%further decomposition
fprintf('\n')
disp('Further Decompositions:')
disp('Decomposing residual into match and transitory component')

xx=M'*M;
xy=M'*r;
m=M*(xx\xy);
e=r-m;
disp('Full Covariance Matrix of Components')
disp('    y      pe      fe      xb      m      e')
C=cov([y,pe,fe,xb,m,e])

clear M

%even further decomposition
disp('Decomposing transitory component into firm/year and person component')
F2=sparse(1:NT,(year-yrmin+1)*J+firmid,1);
F2=F2(:,any(F2,1));
xx=F2'*F2;
xy=F2'*e;
L=ichol(xx,struct('type','ict','droptol',1e-2));
bf=pcg(xx,xy,1e-10,1000,L,L');
f=F2*bf;
e2=e-f;
disp('Full Covariance Matrix of Components')
disp('    y      pe      fe      xb      m      f      e2')
C=cov([y,pe,fe,xb,m,f,e2])

clear F2 xx xy m f e e2

%SAVE OUTPUT (this will write to the current directory)

disp('Saving main effects')
out=[id_old firmid_old year pe fe xb r y birth educ fsize];
s=['AKMeffs' int2str(i) '_norm.txt'];
dlmwrite(s, out, 'delimiter', '\t', 'precision', 16);


clear id_old firmid_old year y pe fe xb r birth educ fsize age S G groupid bf b xx xy beduc bage byear bxb
end
diary off;
